"use strict";
const debug = require('debug')('loadtest');
class BatchOptions {
}
exports.BatchOptions = BatchOptions;
class BatchResults {
}
exports.BatchResults = BatchResults;
class BatchProgress {
}
exports.BatchProgress = BatchProgress;
class Batch {
    constructor(opts) {
        this.opts = opts;
        if (typeof opts.timeout !== 'number' || opts.timeout < 0) {
            opts.timeout = 20000;
        }
        if (typeof opts.iterations !== 'number' || opts.iterations <= 0) {
            opts.iterations = 1;
        }
        if (typeof opts.concurrent !== 'number' || opts.concurrent <= 0) {
            opts.concurrent = 1;
        }
        this.module = require(this.opts.test);
    }
    run(progress) {
        let results = {
            executed: 0,
            succeeded: 0,
            failed: 0,
            totDuration: 0,
            minDuration: 0,
            maxDuration: 0,
            avgDuration: 0,
            results: [],
            errors: []
        };
        return this.module(...(this.opts.args || [])).then((test) => {
            return new Promise(resolve => {
                let runnersLeft = this.opts.iterations;
                let runnersActive = 0;
                let runner = () => {
                    if (!runnersLeft) {
                        results.avgDuration = results.totDuration / results.executed;
                        return Promise.resolve();
                    }
                    else {
                        runnersLeft--;
                        runnersActive++;
                        debug(`starting new runner, left ${runnersLeft}, active ${runnersActive}`);
                        let r;
                        if (test.beforeEach) {
                            r = test.beforeEach();
                        }
                        else {
                            r = Promise.resolve();
                        }
                        return r.then(() => new Promise((resolve, reject) => {
                            let start = new Date();
                            let t = setTimeout(() => reject(new Error('timeout')), this.opts.timeout);
                            let info = {
                                executed: 0,
                                succeeded: 0,
                                failed: 0,
                                totDuration: 0,
                                curDuration: 0
                            };
                            test.run().then(result => {
                                info.result = result;
                                results.succeeded++;
                                results.results.push(result);
                            }, err => {
                                info.error = err;
                                results.failed++;
                                results.results.push(err);
                            }).then(() => {
                                clearTimeout(t);
                                runnersActive--;
                                results.executed++;
                                let end = new Date();
                                let dur = +end - +start;
                                debug(`completed runner in ${dur / 1000}s, left ${runnersLeft}, active ${runnersActive}`);
                                results.totDuration += dur;
                                if (!results.minDuration || dur < results.minDuration) {
                                    results.minDuration = dur;
                                }
                                if (!results.maxDuration || dur > results.maxDuration) {
                                    results.maxDuration = dur;
                                }
                                if (progress) {
                                    info.executed = results.executed;
                                    info.succeeded = results.succeeded;
                                    info.failed = results.failed;
                                    info.totDuration = results.totDuration;
                                    info.curDuration = dur;
                                    progress(info);
                                }
                            }).then(resolve, reject);
                        }).then(() => {
                            if (test.afterEach) {
                                return test.afterEach();
                            }
                        })).then(runner);
                    }
                };
                let concurrent = this.opts.concurrent || 1;
                if (concurrent === 1) {
                    resolve(runner());
                }
                else {
                    let runners = [];
                    for (let i = 0; i < concurrent; i++) {
                        runners.push(runner());
                    }
                    resolve(Promise.all(runners));
                }
            }).then(() => { }, err => {
                results.errors.push(err);
            }).then(() => {
                if (test.dispose) {
                    return test.dispose();
                }
            });
        }).then(() => { }, err => {
            results.errors.push(err);
        }).then(() => results);
    }
}
exports.Batch = Batch;
//# sourceMappingURL=batch.js.map